from pydantic import Field, root_validator
from typing import List, Union, Optional
from models.objects import APObject
from .fields import (
    ActorField,
    InstrumentField,
    ObjectField,
    OriginField,
    ResultField,
    TargetField,
    TypeField,
    AnyOfField, 
    ClosedField, 
    OneOfField
)


class APActivity(APObject):
    actor: Union[ActorField, List[ActorField]]
    object: Union[ObjectField, List[ObjectField]]
    target: Union[TargetField, List[TargetField]]
    origin: Union[OriginField, List[OriginField]]
    result: Union[ResultField, List[ResultField]]
    instrument: Union[InstrumentField, List[InstrumentField]]

class APIntransitiveActivity(APActivity):
    object = None

class APIgnore(APActivity):
    type = TypeField = Field("Ignore", const=True)

class APOffer(ActorField):
    type = TypeField = Field("Offer", const=True)

class APAccept(APActivity):
    type: TypeField = Field("Accept", const=True)

class APAdd(APActivity):
    type: TypeField = Field("Add", const=True)

class APAnnounce(APActivity):
    type: TypeField = Field("Announce", const=True)

class APArrivve(APIntransitiveActivity):
    type: TypeField = Field("Arrive", const=True)

class APBlock(APIgnore):
    type: TypeField = Field("Block", const=True)

class APCreate(APActivity):
    type: TypeField = Field("Create", const=True)

class APDelete(APActivity):
    type: TypeField = Field("Delete", const=True)

class APDislike(APActivity):
    type: TypeField = Field("Dislike", const=True)

class APFlag(APActivity):
    type: TypeField = Field("Flag", const=True)

class APFollow(APActivity):
    type: TypeField = Field("Follow", const=True)

class APInvite(APOffer):
    type: TypeField = Field("Invite", const=True)

class APJoin(APActivity):
    type: TypeField = Field("Join", const=True)

class APLeave(APActivity):
    type: TypeField = Field("Leave", const=True)

class APLike(APActivity):
    type: TypeField = Field("Like", const=True)

class APListen(APActivity):
    type: TypeField = Field("Listen", const=True)

class APMove(APActivity):
    type: TypeField = Field("Move", const=True)

class APRead(APActivity):
    type: TypeField = Field("Read", const=True)

class APReject(APActivity):
    type: TypeField = Field("Reject", const=True)

class APRemove(APActivity):
    type: TypeField = Field("Remove", const=True)

class APTentativeAccept(APActivity):
    type: TypeField = Field("TentativeAccept", const=True)

class APTentativeReject(APActivity):
    type: TypeField = Field("TentativeReject", const=True)

class APTravel(APActivity):
    type: TypeField = Field("Travel", const=True)

class APUndo(APActivity):
    type: TypeField = Field("Undo", const=True)

class APUpdate(APActivity):
    type: TypeField = Field("Update", const=True)

class APView(APActivity):
    type: TypeField = Field("View", const=True)

class Question(APIntransitiveActivity):
    type: TypeField = Field("Question", const=True)
    oneOf: Optional[List[OneOfField]] = None
    anyOf: Optional[List[AnyOfField]] = None
    closed: Optional[ClosedField] = None

    @root_validator(pre=True)
    def check_any_of_or_one_of(cls, values):
        """Ensure that either 'anyOf' or 'oneOf' is used, but not both."""
        if values.get('anyOf') and values.get('oneOf'):
            raise ValueError("A Question object must not have both 'anyOf' and 'oneOf'.")
        return values

class APQuestionAnyOf(Question):
    oneOf: None

class APQuestionOneOf(Question):
    anyOf: None 

class APQuestionNone(Question):
    oneOf: None
    anyOf: None
