from __future__ import annotations
from pydantic import Field
from typing import Optional
from .objects import APObject
from .fields import (
    FollowersField,
    FollowingField,
    InboxField,
    LikedField,
    OutboxField,
    PreferredUsernameMapField,
    PreferredUsernameField,
    StreamsField,
    EndpointsField,
    TypeField
)

class APActor(APObject):
    """
    Base class for ActivityPub actors.
    """
    inbox: InboxField
    outbox: OutboxField
    following: Optional[FollowingField] = None
    followers: Optional[FollowersField] = None
    liked: Optional[LikedField] = None
    streams: Optional[StreamsField] = None
    preferred_username: Optional[PreferredUsernameField] = None
    preferred_username_map: Optional[PreferredUsernameMapField] = None
    endpoints: Optional[EndpointsField] = None

class APPerson(APActor):
    """
    Represents a Person actor in ActivityPub.
    """
    type: TypeField = Field("Person", const=True)

class APGroup(APActor):
    """
    Represents a Group actor in ActivityPub.
    """
    type: TypeField = Field("Group", const=True)

class APOrganization(APActor):
    """
    Represents an Organization actor in ActivityPub.
    """
    type: TypeField = Field("Organization", const=True)

class APApplication(APActor):
    """
    Represents an Application actor in ActivityPub.
    """
    type: TypeField = Field("Application", const=True)

class APService(APActor):
    """
    Represents a Service actor in ActivityPub.
    """
    type: TypeField = Field("Service", const=True)