from __future__ import annotations
from typing import Optional, List, Union
from pydantic import Field
from .fields import (
    CollectionCurrentField,
    CollectionFirstField, CollectionItemsField,
    CollectionLastField,
    TotalItemsField,
    CollectionPageNextField, 
    CollectionPagePartOfField, 
    CollectionPagePrevField,
    StartIndexField,
    OrderedCollectionItemsField,
    TypeField
)
from .objects import APObject

class APCollection(APObject):
    """
    Collection object, inheriting from Object

    """
    type: TypeField = Field("Collection", const=True)
    items: List[CollectionItemsField]
    total_items: Optional[TotalItemsField] = None
    first: Optional[CollectionFirstField] = None
    last: Optional[CollectionLastField] = None
    current: Optional[CollectionCurrentField] = None

class APCollectionPage(APCollection):
    type: TypeField = Field("CollectionPage", const=True) 
    part_of: Optional[CollectionPagePartOfField] = None
    next: Optional[CollectionPageNextField] = None
    prev: Optional[CollectionPagePrevField] = None

class APOrderedCollection(APCollectionPage):
    type: TypeField = Field("OrderedCollectionPage", const=True)
    ordered_items: OrderedCollectionItemsField

class APOrderedCollectionPage(APCollection, APCollectionPage):
    type: TypeField = Field("OrderedCollectionPage", const=True)
    start_index: StartIndexField